/****************************************************************************
**
** DocToText - Converts MS Word, RTF, ODF and OOXML documents to plain text
** Copyright (c) 2006-2009, SILVERCODERS(R)
** http://silvercoders.com
**
** Project homepage: http://silvercoders.com/index.php?page=DocToText
**
** This program may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file COPYING.GPL included in the
** packaging of this file.
**
** Please remember that any attempt to workaround the GNU General Public
** License using wrappers, pipes, client/server protocols, and so on
** is considered as license violation. If your program, published on license
** other than GNU General Public License version 2, calls some part of this
** code directly or indirectly, you have to buy commercial license.
** If you do not like our point of view, simply do not use the product.
**
** Licensees holding valid commercial license for this product
** may use this file in accordance with the license published by
** SILVERCODERS and appearing in the file COPYING.COM
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
**
*****************************************************************************/

#include "doctotext_rtf_parser.h"
#include "doctotext_odf_ooxml_parser.h"
#include "doctotext_misc.h"
#include "doctotext_unzip.h"
#include <glib/gstring.h>
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <string>
#include "handlers.h"
#include "parser.h"
#include "parserfactory.h"
#include "ustring.h"
#include "../version.h"
using namespace std;
using namespace wvWare;

class DocToTextHandler : public TextHandler
{
	private:
		UString* Text;

	public:
		DocToTextHandler(UString* text)
		{
			Text = text;
		}

		void sectionStart(SharedPtr<const Word97::SEP> sep)
		{
		}

		void sectionEnd()
		{
		}

		void pageBreak()
		{
		}

		void headersFound(const HeaderFunctor &parseHeaders)
		{
		}

		void paragraphStart(SharedPtr<const ParagraphProperties>
			paragraphProperties)
		{
			(*Text) += UString("\n");
		}

		void paragraphEnd()
		{
			(*Text) += UString("\n");
		}

		void runOfText (const UString &text, SharedPtr< const Word97::CHP > chp)
		{
			(*Text) += text;
		}

		void specialCharacter(SpecialCharacter character,
			SharedPtr<const Word97::CHP> chp)
		{
		}

		void footnoteFound (FootnoteData::Type type, UChar character,
			SharedPtr<const Word97::CHP> chp,
			const FootnoteFunctor &parseFootnote)
		{
		}

		void footnoteAutoNumber(SharedPtr<const Word97::CHP> chp)
		{
		}

		void fieldStart(const FLD *fld,
			SharedPtr<const Word97::CHP> chp)
		{
		}

		void fieldSeparator(const FLD* fld,
			SharedPtr<const Word97::CHP> chp)
		{
		}

		void fieldEnd(const FLD* fld,
			SharedPtr<const Word97::CHP> chp)
		{
		}
};

static void version()
{
	printf("DocToText v%s\nConverts MS Word, RTF, ODF and OOXML documents to plain text\nCopyright (c) 2006-2009 SILVERCODERS(R)\nhttp://silvercoders.com\n", VERSION);
}

static void help()
{
	version();
	printf("\nUsage: doctotext [OPTION]... [FILE]\n\n");
	printf("FILE\t\tname of file to convert\n\n");
	printf("Options:\n");
	printf("--fix-xml\ttry to fix corrupted xml files (ODF, OOXML)\n");
	printf("--strip-xml\tstrip xml tags instead of parsing them (ODF, OOXML)\n");
	printf("--unzip-cmd=[COMMAND]\tuse COMMAND to unzip files from archives (ODF, OOXML)\n"
		"\tinstead of build-in decompression library.\n"
		"\t%%d in the command is substituted with destination directory path\n"
		"\t%%a in the command is substituted with name of archive file\n"
		"\t%%f in the command is substituted with name of file to extract\n"
		"\tExample: --unzip-cmd=\"unzip -d %%d %%a %%f\"\n"
	);
}

static void print_ustring(const UString& s)
{
	gchar out[6];
	for (int i = 0; i < s.length(); i++)
	{
		gint len = g_unichar_to_utf8(s[i].unicode(), out);
		out[len] = '\0';
		printf("%s", out);
	}
}

static int print_doc_file(const char* file_name)
{
	SharedPtr<Parser> parser = ParserFactory::createParser(file_name);
	if (!parser || !parser->isOk())
	{
		fprintf(stderr, "Creating parser failed.\n");
		return EXIT_FAILURE;
	}
	UString text;
	DocToTextHandler handler(&text);
	parser->setTextHandler(&handler);
	if (!parser->parse())
	{
		fprintf(stderr, "Parsing document failed.\n");
		return EXIT_FAILURE;
	}
	print_ustring(text);
	printf("%s", "\n");
	return EXIT_SUCCESS;
}

int main(int argc, char* argv[])
{
	if (argc < 2)
	{
		help();
		return EXIT_FAILURE;
	}

	std::string arg;
	std::string cmd;

	ODFOOXMLParser::ParserMode mode = ODFOOXMLParser::PARSE_XML;

	FormattingStyle options;	
	options.table_style = TABLE_STYLE_ONE_COL;
	options.list_style = LIST_STYLE_NUMBER;
	options.url_style = URL_STYLE_EXTENDED;

	for(int i = 1 ; i < argc ; i ++)
	{
		arg = argv[i-1];

		if(arg.find("table-style=", 0) != -1)
		{
			if(arg.find("one-row", arg.find("table-style=", 0) + 11) != -1)
			{
				options.table_style = TABLE_STYLE_ONE_ROW;
			}
			if(arg.find("one-col", arg.find("table-style=", 0) + 11) != -1)
			{
				options.table_style = TABLE_STYLE_ONE_COL;
			}
			if(arg.find("table-look", arg.find("table-style=", 0) + 11) != -1)
			{
				options.table_style = TABLE_STYLE_TABLE_LOOK;
			}
		}
		if(arg.find("url-style=", 0) != -1)
		{
			if(arg.find("text-only", arg.find("url-style=", 0) + 10) != -1)
			{
				options.url_style = URL_STYLE_TEXT_ONLY;
			}
			if(arg.find("extended", arg.find("url-style=", 0) + 10) != -1)
			{
				options.url_style = URL_STYLE_EXTENDED;
			}
		}
		if(arg.find("list-style=", 0) != -1)
		{
			if(arg.find("number", arg.find("list-style=", 0) + 11) != -1)
			{
				options.list_style = LIST_STYLE_NUMBER;
			}
			if(arg.find("dash", arg.find("list-style=", 0) + 11) != -1)
			{
				options.list_style = LIST_STYLE_DASH;
			}
			if(arg.find("dot", arg.find("list-style=", 0) + 11) != -1)
			{
				options.list_style = LIST_STYLE_DOT;
			}
		}
		if (arg.find("fix-xml", 0) != std::string::npos)
			mode = ODFOOXMLParser::FIX_XML;
		if (arg.find("strip-xml", 0) != std::string::npos)
			mode = ODFOOXMLParser::STRIP_XML;
		if (arg.find("unzip-cmd=", 0) != -1)
		{
			DocToTextUnzip::setUnzipCommand(arg.substr(arg.find("unzip-cmd=", 0) + 10));
		}
	}

	RTFParser rtf(argv[argc - 1]);
	bool is_rtf = rtf.isRTF();
	if (rtf.error())
		return EXIT_FAILURE;
	if (is_rtf)
	{
		fprintf(stderr, "Using RTF parser.\n");
		UString text = rtf.plainText();
		if (rtf.error())
			return EXIT_FAILURE;
		print_ustring(text);
		printf("%s", "\n");
		return EXIT_SUCCESS;
	}
	else
	{
		ODFOOXMLParser odfooxml(argv[argc - 1]);
		bool is_odfooxml = odfooxml.isODFOOXML();
		if (odfooxml.error())
			return EXIT_FAILURE;
		if (is_odfooxml)
		{
			fprintf(stderr, "Using ODF/OOXML parser.\n");
			string text = odfooxml.plainText(mode, options);
			if (odfooxml.error())
				return EXIT_FAILURE;
			printf("%s\n", text.c_str());
			return EXIT_SUCCESS;
		}
		else
		{
			fprintf(stderr, "Using DOC parser.\n");
			return print_doc_file(argv[argc - 1]);
		}
	}
}
